Imports Vb2008PinvokeExample.DeVaSys.UsbI2cIo.VB.Pinvoke

Public Class frmMain
  ' This form provides a demonstration of each function call provided by the
  ' DeVaSys USBI2CIO Device Application Programming Interface (DAPI)

Dim sbOutput As New System.Text.StringBuilder("", 2048)
'
Public hDevInstance As IntPtr
Public sDevSymName As String                                ' used for symbolic name of USB device "UsbI2cIo"
Public INVALID_HANDLE_VALUE As IntPtr = CType(-1, IntPtr)   ' this should really be a constant, but VB won't allow constants of type IntPtr


Private Sub frmMain_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
  sDevSymName = "UsbI2cIo"                      ' initial device symbolic name
  hDevInstance = INVALID_HANDLE_VALUE           ' initialize file handle
'  DoAll()
End Sub

  Public Sub CloseDiHandle()
    If hDevInstance <> INVALID_HANDLE_VALUE Then
      DAPI.CloseDeviceInstance(hDevInstance)
      hDevInstance = INVALID_HANDLE_VALUE
    End If
  End Sub

  Public Function OpenDiHandle() As Boolean
    ' this subroutine will handle opening the current device instance

    ' make sure no handle is currently open
    CloseDiHandle()

    ' now attempt to open handle to device instance
    hDevInstance = DAPI.OpenDeviceInstance(sDevSymName, 0)

    ' test result of function call and flag success or failure
    If (hDevInstance <> INVALID_HANDLE_VALUE) Then
      OpenDiHandle = True
    Else
      OpenDiHandle = False
    End If
  End Function
'
Private Sub DoAll()

  sbOutput.Length = 0
  DemoGetDllAndDriverVersions()
  sbOutput.AppendLine()
  If (DemoGetDeviceCount() > 0) Then
    sbOutput.AppendLine()
    DemoGetDeviceInfo()
    sbOutput.AppendLine()
    If (OpenDiHandle()) Then
      sbOutput.AppendLine("Opened handle to UsbI2cIo board")
      sbOutput.AppendLine()
      DemoGetSerialId()
      sbOutput.AppendLine()
      DemoIo()
      sbOutput.AppendLine()
      DemoI2c()
      sbOutput.AppendLine()
      DemoProperties()
      sbOutput.AppendLine()
      DemoGetVendorRequest()
      sbOutput.AppendLine()
      DemoGetAndClearError()
      sbOutput.AppendLine()
      CloseDiHandle()
      sbOutput.AppendLine("Closed handle to UsbI2cIo board")
    Else
      sbOutput.AppendLine("Error: Failed to open handle to UsbI2cIo board")
    End If
  End If
  txtSbOut.Text = sbOutput.ToString
End Sub

'
Private Function DemoDetectDevice() As Boolean

  sbOutput.AppendLine("DemoDetectDevice")

  If (DAPI.DetectDevice(hDevInstance)) Then
    sbOutput.AppendLine("UsbI2cIo device detected")
    DemoDetectDevice = True
  Else
    sbOutput.AppendLine("UsbI2cIo device not detected")
    DemoDetectDevice = False
  End If

End Function
'
Private Function DemoGetDeviceCount() As Byte

  Dim byCount As Byte

  sbOutput.AppendLine("DemoGetDeviceCount")

  byCount = DAPI.GetDeviceCount(sDevSymName)
  sbOutput.Append("Attached UsbI2cIo Device(s) = ")
  sbOutput.AppendLine(byCount.ToString)
  DemoGetDeviceCount = byCount

End Function
'
Private Sub DemoGetDllAndDriverVersions()

  Dim Version As UInt16

  sbOutput.AppendLine("DemoGetDllAndDriverVersions")

  Version = DAPI.GetDllVersion()
  sbOutput.AppendFormat("UsbI2cio DLL Version    = {0:X}.{1:X2}", CByte(Version / 256), CByte(Version And 255))
  sbOutput.AppendLine()

  If Version < &H304 Then
    MsgBox("Warning: You must use version 3.04 (or later) of usbi2cio.dll file in order to properly fetch serial ID from version C USB-I2C/IO boards", _
    , _
    "Warning: Old version of UsbI2cIo.dll detected!")
  End If

  Version = DAPI.GetDriverVersion()
  sbOutput.AppendFormat("UsbI2cio Driver Version = {0:X}.{1:X2}", CByte(Version / 256), CByte(Version And 255))
  sbOutput.AppendLine()

End Sub
'
Private Sub DemoGetFirmwareVersion()

  Dim Version As UInt16

  sbOutput.AppendLine("DemoGetFirmwareVersion")

  If (DAPI.GetFirmwareVersion(hDevInstance, Version)) Then
    sbOutput.AppendFormat("    Firmware Version = {0:X}.{1:X2}", CByte(Version / 256), CByte(Version And 255))
    sbOutput.AppendLine()
  Else
    sbOutput.AppendLine("    Error: Firmware Version = Unavailable")
  End If

End Sub
'
Private Sub DemoGetSerialId()

  Dim sbSerialId As System.Text.StringBuilder = New System.Text.StringBuilder("", 9)

  sbOutput.AppendLine("DemoGetSerialId")

'  If (DAPI_GetSerialId(modMain.hDevInstance, Id)) Then
  If (DAPI.GetSerialId(hDevInstance, sbSerialId)) Then
    sbOutput.Append("Serial ID = ")
'    sbOutput.AppendLine(Id.sSerial)
    sbOutput.AppendLine(sbSerialId.ToString)
  Else
    sbOutput.AppendLine("Error: Serial ID = Unavalable")
  End If

End Sub
'
Private Sub DemoGetDeviceInfo()

  Dim DeviceInfoArray As DAPI.DEV_INFO_ARRAY = New DAPI.DEV_INFO_ARRAY
  Dim byCount As Byte = 0
  Dim i As Integer

  sbOutput.AppendLine("DemoGetDeviceInfo")

  byCount = DAPI.GetDeviceInfo(sDevSymName, DeviceInfoArray)
  If (byCount > 0) Then
    sbOutput.AppendLine("Device Info:")
    For i = 0 To (byCount - 1) Step 1
      sbOutput.Append("  Device Instance = ")
      sbOutput.Append(DeviceInfoArray.DevInfo(i).byInstance.ToString)
      sbOutput.Append(", SerialId = ")
      sbOutput.AppendLine(DeviceInfoArray.DevInfo(i).SerialId.ToString)
      DemoOpenDeviceBySerialId(DeviceInfoArray.DevInfo(i).SerialId)
    Next i
  Else
    sbOutput.AppendLine("Error: Device Info = Unavalable")
  End If

End Sub

'
Private Function DemoOpenDeviceBySerialId(ByVal sSerial As String) As Boolean

  sbOutput.AppendLine("DemoOpenDeviceBySerialId")

  ' now let's close our handle
   CloseDiHandle()

  ' get a handle to a board, using the SerialId of the board
  hDevInstance = DAPI.OpenDeviceBySerialId(sDevSymName, sSerial)

  ' test result of function call and flag success or failure
  If (hDevInstance <> INVALID_HANDLE_VALUE) Then
    sbOutput.Append("    Opened handle to device using Serial ID ")
    sbOutput.AppendLine(sSerial.ToString)
    DemoGetFirmwareVersion()
    DemoOpenDeviceBySerialId = True
  Else
    sbOutput.Append("    Error: Failed to open device using Serial ID ")
    sbOutput.AppendLine(sSerial.ToString)
    DemoOpenDeviceBySerialId = False
  End If

End Function
'
Private Sub DemoIo()

  Dim IoPortConfig As UInt32
  Dim IoPortMask As UInt32
  Dim IoPortData As UInt32

  sbOutput.AppendLine("DemoIo")

  If (DAPI.GetIoConfig(hDevInstance, IoPortConfig)) Then
    sbOutput.AppendFormat("Read I/O Configuration 0x{0:X8} from UsbI2cIo board", IoPortConfig)
    sbOutput.AppendLine()
  Else
    sbOutput.AppendLine("Error I/O Configuration = Unavalable")
  End If

  IoPortConfig = &H0
  If (DAPI.ConfigIoPorts(hDevInstance, IoPortConfig)) Then
    sbOutput.AppendFormat("Wrote I/O Configuration 0x{0:X8} to UsbI2cIo board", IoPortConfig)
    sbOutput.AppendLine()
  Else
    sbOutput.AppendLine("Error: Failed to write I/O Configuration to UsbI2cIo board")
  End If


  If (DAPI.GetIoConfig(hDevInstance, IoPortConfig)) Then
    sbOutput.AppendFormat("Read I/O Configuration 0x{0:X8} from UsbI2cIo board", IoPortConfig)
    sbOutput.AppendLine()
  Else
    sbOutput.AppendLine("Error I/O Configuration = Unavalable")
  End If

  If (DAPI.ReadIoPorts(hDevInstance, IoPortData)) Then
    sbOutput.AppendFormat("Read I/O Data 0x{0:X8} from UsbI2cIo board", IoPortData)
    sbOutput.AppendLine()
  Else
    sbOutput.AppendLine("Error: I/O Data = Unavalable")
  End If

'  IoPortMask = &HFFFFFFFF ' this produces an error
  IoPortMask = 4294967295 ' have to do it this way because VB oddity
  IoPortData = &H1
  If (DAPI.WriteIoPorts(hDevInstance, IoPortData, IoPortMask)) Then
    sbOutput.AppendFormat("Wrote I/O Data 0x{0:X8}, I/O Mask 0x{1:X8} to UsbI2cIo board", IoPortData, IoPortMask)
    sbOutput.AppendLine()
  Else
    sbOutput.AppendLine("Error: Failed to write I/O Data and I/O Mask to UsbI2cIo board")
  End If

  If (DAPI.ReadIoPorts(hDevInstance, IoPortData)) Then
    sbOutput.AppendFormat("Read I/O Data 0x{0:X8} from UsbI2cIo board", IoPortData)
    sbOutput.AppendLine()
  Else
    sbOutput.AppendLine("Error: I/O Data = Unavalable")
  End If

End Sub
'
Private Sub DemoI2c()

  Dim I2cTrans As DAPI.I2C_TRANS = New DAPI.I2C_TRANS
  Dim lResult As Int32
  Dim i As Integer

  sbOutput.AppendLine("DemoI2c")

  I2cTrans.bySlvDevAddr = &HA2
  I2cTrans.byTransType = DAPI.I2C_TRANS_TYPE.ADR_16
  I2cTrans.wMemoryAddr = &H3F00
  'I2cTrans.wMemAddr.lo = &H0
  I2cTrans.wCount = 8

  lResult = DAPI.ReadI2c(hDevInstance, I2cTrans)
  sbOutput.AppendLine("Setting up I2C_TRANS structure for call to ReadI2c():")
  sbOutput.AppendFormat("  I2cTrans.byDevId: 0x{0:X2}", I2cTrans.bySlvDevAddr)
  sbOutput.AppendLine()
  sbOutput.AppendFormat("  I2cTrans.byType: 0x{0:X2}", I2cTrans.byTransType)
  sbOutput.AppendLine()
  sbOutput.AppendFormat("  I2cTrans.wMemoryAddr: 0x{0:X4}", I2cTrans.wMemoryAddr)
  sbOutput.AppendLine()
  sbOutput.AppendFormat("  I2cTrans.wCount: 0x{0:X4}", I2cTrans.wCount)
  sbOutput.AppendLine()

  If (lResult = I2cTrans.wCount) Then
    sbOutput.AppendLine("ReadI2c call was successful")
    sbOutput.AppendFormat("ReadI2c call was successful, return value = 0x{0:X8}", lResult)
    sbOutput.AppendLine()
    sbOutput.Append("  I2cTrans.Data(): ")
    For i = 0 To (lResult - 1) Step 1
      sbOutput.AppendFormat(" 0x{0:X2}", I2cTrans.Data(i))
    Next i
    sbOutput.AppendLine(" }")
  Else
    sbOutput.AppendFormat("Error: ReadI2c call failed, return value = 0x{0:X8}\r\n", lResult)
  End If

End Sub
'
Public Sub DemoProperties()

  Dim byPropertyCount As Byte
  Dim byProperty As Byte
  Dim i As Integer
'  Dim lValue As Long
'  Dim iValue As Integer
'  Dim byValue As Byte

  sbOutput.AppendLine("DemoProperties")

  ' Get Property Table Size
  If (DAPI.GetProperty(hDevInstance, byPropertyCount, DAPI.PROPERTY_IDX.WRCOMMAND_RDCOUNT)) Then
    sbOutput.AppendFormat("Property Count: 0x{0:X2} ({1})", byPropertyCount, byPropertyCount)
    sbOutput.AppendLine()


  ' Get all properties
    For i = 0 To (byPropertyCount - 1)
      If (DAPI.GetProperty(hDevInstance, byProperty, CByte(i))) Then
        sbOutput.AppendFormat("Property 0x{0:X2} ({1})", byProperty, byProperty)
        sbOutput.AppendLine()
      Else
        sbOutput.AppendLine("Error: GetProperty call failed")
      End If
    Next i

  ' Set a property
'  If (DAPI_SetProperty(modMain.hDevInstance, PROPERTY_WRCOMMAND_RDCOUNT, PROPERTY_CMD_RESET_TO_DEFAULTS)) Then
'    If (DAPI_SetProperty(modMain.hDevInstance, PROPERTY_WRCOMMAND_RDCOUNT, PROPERTY_CMD_STORE_TABLE_TO_EEPROM)) Then
'    End If
'  End If

  ' Make current properties non-volatile (store them in flash)
  ' Invalidate flash copy of properties (restores default properties)
  Else
    sbOutput.AppendLine("Error: GetProperty call failed")
  End If

End Sub
'
Public Sub DemoGetVendorRequest()

  Dim VrData() As Byte = New Byte(DAPI.VR_MAX_DATA) {}
  Dim byRequest As Byte = &HF2
  Dim wValue As UInt16 = &HA202
  Dim wIndex As UInt16 = &H0
  Dim wLength As UInt16 = &H400
  Dim i As Integer

  sbOutput.AppendLine("DemoVendorRequest")

  'bmRequestType = Device to Host (Get), Vendor Request 
  'bRequest = 0xF2
  'wValue = I2C_TRANS.byTransType (byte 2) and I2C_TRANS.bySlvDevAddr (byte 3)
  'wIndex = I2C_TRANS.wMemoryAddr
  'wLength = I2C_TRANS.wCount

  If (DAPI.GetVendorRequest(hDevInstance, VrData(0), byRequest, wValue, wIndex, wLength) = wLength) Then
    sbOutput.AppendLine("GetVendorRequest call was successful")
    sbOutput.Append("Data(): {")
    For i = 0 To (wLength - 1) Step 1
    sbOutput.AppendFormat(" 0x{0:X2}", VrData(i))
    Next i
    sbOutput.AppendLine(" }")
  Else
    sbOutput.AppendLine("Error: GetVendorRequest call failed")
  End If

End Sub
'
Public Sub DemoGetAndClearError()

  Dim iError As Int32

  sbOutput.AppendLine("DemoGetAndClearError")


  If (DAPI.GetLastFirmwareError(hDevInstance, iError)) Then
    sbOutput.AppendFormat("GetLastFimrwareError call returned: 0x{0:X8}", iError)
    sbOutput.AppendLine()
    ' only least significant byte of returned unsigned integer is used, need to strip upper bytes
    If ((iError And &HFF) <> 0) Then
      sbOutput.AppendFormat("Last Firmware Error : 0x{0:X2}", iError And &HFF)
      sbOutput.AppendLine()
    Else
      sbOutput.AppendLine("No Firmware Error : 0x00")
    End If
  Else
    sbOutput.AppendLine("Error: GetLastFirmwareError call failed")
  End If

End Sub
'
Private Sub btnDoAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDoAll.Click
  DoAll()
End Sub
'
Private Sub btnClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClear.Click
  sbOutput.Length = 0
  txtSbOut.Text = sbOutput.ToString
End Sub
'
Private Sub btnVersion_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnVersion.Click
  DemoGetDllAndDriverVersions()
  sbOutput.AppendLine()
  If (DemoGetDeviceCount() > 0) Then
    DemoGetDeviceInfo()
  End If
  CloseDiHandle()    ' now let's close our handle
  sbOutput.AppendLine()
  txtSbOut.Text = sbOutput.ToString
End Sub
'
Private Sub btnIo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnIo.Click
  If (OpenDiHandle()) Then
    sbOutput.AppendLine("Opened handle to UsbI2cIo board")
    DemoIo()
    CloseDiHandle()
    sbOutput.AppendLine("Closed handle to UsbI2cIo board")
  Else
    sbOutput.AppendLine("Error: Failed to open handle to UsbI2cIo board")
  End If
  sbOutput.AppendLine()
  txtSbOut.Text = sbOutput.ToString
End Sub
'
Private Sub btnI2c_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnI2c.Click
  If (OpenDiHandle()) Then
    sbOutput.AppendLine("Opened handle to UsbI2cIo board")
    DemoI2c()
    CloseDiHandle()
    sbOutput.AppendLine("Closed handle to UsbI2cIo board")
  Else
    sbOutput.AppendLine("Error: Failed to open handle to UsbI2cIo board")
  End If
  sbOutput.AppendLine()
  txtSbOut.Text = sbOutput.ToString
End Sub
'
Private Sub btnProperties_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnProperties.Click
  If (OpenDiHandle()) Then
    sbOutput.AppendLine("Opened handle to UsbI2cIo board")
    DemoProperties()
    CloseDiHandle()
    sbOutput.AppendLine("Closed handle to UsbI2cIo board")
  Else
    sbOutput.AppendLine("Error: Failed to open handle to UsbI2cIo board")
  End If
  sbOutput.AppendLine()
  txtSbOut.Text = sbOutput.ToString
End Sub
'
Private Sub btnVenReqs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnVenReqs.Click
  If (OpenDiHandle()) Then
    sbOutput.AppendLine("Opened handle to UsbI2cIo board")
    DemoGetVendorRequest()
    CloseDiHandle()
    sbOutput.AppendLine("Closed handle to UsbI2cIo board")
  Else
    sbOutput.AppendLine("Error: Failed to open handle to UsbI2cIo board")
  End If
  sbOutput.AppendLine()
  txtSbOut.Text = sbOutput.ToString
End Sub
'
Private Sub btnGetClrError_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGetClrError.Click
  If (OpenDiHandle()) Then
    sbOutput.AppendLine("Opened handle to UsbI2cIo board")
    DemoGetAndClearError()
    CloseDiHandle()
    sbOutput.AppendLine("Closed handle to UsbI2cIo board")
  Else
    sbOutput.AppendLine("Error: Failed to open handle to UsbI2cIo board")
  End If
  sbOutput.AppendLine()
  txtSbOut.Text = sbOutput.ToString
End Sub
'
Private Sub cmdExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdExit.Click
  Me.Close()
End Sub

'
Public Function HexN(ByVal lValue As Long, Optional ByVal iWidth As Integer = 0, Optional ByVal bPrefix As Boolean = False) As String
  If (iWidth <= 0) Then
    ' no zero padding
    If bPrefix Then
      HexN = "0x" & Hex$(lValue)
    Else
      HexN = Hex$(lValue)
    End If

  Else
    ' need to add specified amount of left zero padding
    If bPrefix Then
      HexN = "0x" & Microsoft.VisualBasic.Right$((StrDup$(iWidth, "0") & Hex$(lValue)), iWidth)
    Else
      HexN = Microsoft.VisualBasic.Right$((StrDup$(iWidth, "0") & Hex$(lValue)), iWidth)
    End If
  End If
End Function

End Class